package br.com.indzooom.indzooom_android_downloader

import android.Manifest
import android.app.Activity
import android.app.DownloadManager
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Environment

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

/**
 * IndzooomAndroidDownloaderPlugin
 */
class IndzooomAndroidDownloaderPlugin internal constructor(internal var activity: Activity) : MethodCallHandler {

    internal val PERMISSION_CODE = 1000
    internal var url: String
    internal var fileName: String
    internal var directory: String

    @Override
    fun onMethodCall(call: MethodCall, result: Result) {
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + Build.VERSION.RELEASE)
        } else if (call.method.equals("getPermission")) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (activity.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) === PackageManager.PERMISSION_DENIED) {
                    val permissions = arrayOf<String>(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                    activity.requestPermissions(permissions, PERMISSION_CODE)
                }
            }
        } else if (call.method.equals("Download")) {
            url = call.argument("url")
            fileName = call.argument("fileName")
            directory = call.argument("directory")

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (activity.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) === PackageManager.PERMISSION_DENIED) {
                    val permissions = arrayOf<String>(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                    activity.requestPermissions(permissions, PERMISSION_CODE)

                } else {
                    startDownloading()
                }
            } else {
                startDownloading()
            }
        }
    }

    fun startDownloading() {
        val request = DownloadManager.Request(Uri.parse(url))
        request.allowScanningByMediaScanner()
        Environment.getExternalStoragePublicDirectory(directory).mkdir()
        request.setDestinationInExternalPublicDir(directory, fileName)
        // Set the download network type // 设置下载网络类型
        request.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_WIFI or DownloadManager.Request.NETWORK_MOBILE)
        // set notification title and description //设置 Notification 的标题和描述
        request.setTitle(fileName)
        request.setDescription("Baixando arquivos")
        // Set the Notification display, and hide. //设置 Notification 的显示，和隐藏。
        request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED)
        val manager = activity.getSystemService(activity.DOWNLOAD_SERVICE) as DownloadManager
        manager.enqueue(request)
    }

    companion object {
        /**
         * Plugin registration.
         */
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "indzooom_android_downloader")
            channel.setMethodCallHandler(IndzooomAndroidDownloaderPlugin(registrar.activity()))
        }
    }
}
